==> An_V1.9.0.tar.gz: kmpdfkit_demo/ContentEditor/src/main/java/com/compdfkit/contenteditor/PDFEditSampleActivity.java <==
/**
 * Copyright © 2014-2023 PDF Technologies, Inc. All Rights Reserved.
 * <p>
 * THIS SOURCE CODE AND ANY ACCOMPANYING DOCUMENTATION ARE PROTECTED BY INTERNATIONAL COPYRIGHT LAW
 * AND MAY NOT BE RESOLD OR REDISTRIBUTED. USAGE IS BOUND TO THE ComPDFKit LICENSE AGREEMENT.
 * UNAUTHORIZED REPRODUCTION OR DISTRIBUTION IS SUBJECT TO CIVIL AND CRIMINAL PENALTIES.
 * This notice may not be removed from this file.
 */
package com.compdfkit.contenteditor;

import static com.compdfkit.tools.contenteditor.CEditToolbar.SELECT_AREA_IMAGE;
import static com.compdfkit.tools.contenteditor.CEditToolbar.SELECT_AREA_TEXT;
import static com.compdfkit.ui.contextmenu.CPDFContextMenuShowHelper.AddEditImageArea;
import static com.compdfkit.ui.contextmenu.CPDFContextMenuShowHelper.ReplaceEditImageArea;

import android.Manifest;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.view.View;
import android.widget.Toast;

import androidx.activity.result.ActivityResultLauncher;
import androidx.activity.result.contract.ActivityResultContracts;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.compdfkit.contenteditor.databinding.EditPdfSampleActivityBinding;
import com.compdfkit.core.edit.CPDFEditManager;
import com.compdfkit.tools.common.activity.CBasicActivity;
import com.compdfkit.tools.common.contextmenu.CPDFContextMenuHelper;
import com.compdfkit.tools.common.utils.CFileUtils;
import com.compdfkit.tools.common.utils.CUriUtil;
import com.compdfkit.tools.common.utils.dialog.CAlertDialog;
import com.compdfkit.tools.common.utils.task.CExtractAssetFileTask;
import com.compdfkit.tools.common.utils.threadpools.CThreadPoolUtils;
import com.compdfkit.tools.common.utils.window.CPopupMenuWindow;
import com.compdfkit.tools.common.views.pdfbota.CPDFBOTA;
import com.compdfkit.tools.common.views.pdfbota.CPDFBotaDialogFragment;
import com.compdfkit.tools.common.views.pdfbota.CPDFBotaFragmentTabs;
import com.compdfkit.tools.common.views.pdfproperties.pdfstyle.CAnnotStyle;
import com.compdfkit.tools.common.views.pdfproperties.pdfstyle.CStyleDialogFragment;
import com.compdfkit.tools.common.views.pdfproperties.pdfstyle.CStyleType;
import com.compdfkit.tools.common.views.pdfproperties.pdfstyle.manager.CStyleManager;
import com.compdfkit.tools.common.views.pdfview.CPreviewMode;
import com.compdfkit.tools.contenteditor.CPDFEditType;
import com.compdfkit.tools.viewer.pdfsearch.CSearchResultDialogFragment;
import com.compdfkit.ui.reader.CPDFPageView;
import com.compdfkit.ui.reader.CPDFReaderView;

import java.util.ArrayList;

import pub.devrel.easypermissions.AfterPermissionGranted;
import pub.devrel.easypermissions.EasyPermissions;

public class PDFEditSampleActivity extends CBasicActivity {

    private final static int RC_PERMISSION_PERM = 111;

    /**
     * assets folder pdf file
     */
    public static final String QUICK_START_GUIDE = "Quick Start Guide V2.4.0.pdf";

    private EditPdfSampleActivityBinding binding;

    private CEditSampleScreenManager screenManager = new CEditSampleScreenManager();

    private ActivityResultLauncher<Intent> selectDocumentLauncher = registerForActivityResult(new ActivityResultContracts.StartActivityForResult(), result -> {
        if (result.getData() != null && result.getData().getData() != null) {
            CPDFReaderView readerView = binding.pdfView.getCPdfReaderView();
            if (readerView != null && readerView.getEditManager() != null) {
                readerView.getEditManager().endEdit();
            }
            Uri uri = result.getData().getData();
            CFileUtils.takeUriPermission(this, uri);
            binding.pdfEditToolBar.resetStatus();
            resetContextMenu(binding.pdfView, CPreviewMode.Edit);
            binding.pdfToolBar.selectMode(CPreviewMode.Edit);
            binding.pdfView.openPDF(uri, () -> {
                binding.pdfEditToolBar.setEditMode(true);
            });
        }
    });

    int curEditMode = 0;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        binding = EditPdfSampleActivityBinding.inflate(getLayoutInflater());
        setContentView(binding.getRoot());
        screenManager.bind(binding);
        initPDFView();
        initToolbarListener();
        initSearchBar();
        initEditBar();
        onDoNext();
    }

    private void initPDFView() {
        binding.pdfView.setAllowAddAndEditAnnot(false);
        resetContextMenu(binding.pdfView, CPreviewMode.Edit);
        binding.pdfView.getCPdfReaderView().setMinScaleEnable(false);
        //Extract PDF files from the Android assets folder
        CExtractAssetFileTask.extract(this, QUICK_START_GUIDE, QUICK_START_GUIDE, (filePath) -> {
            binding.pdfView.openPDF(filePath);
            binding.pdfEditToolBar.setEditMode(true);
        });
        binding.pdfView.setOnTapMainDocAreaCallback(() -> {
            screenManager.fillScreenChange();
        });
        binding.pdfView.getCPdfReaderView().setMinScaleEnable(false);
    }

    private void initToolbarListener() {
        binding.pdfToolBar.addMode(CPreviewMode.Edit);
        binding.pdfToolBar.setSearchBtnClickListener(v -> {
            CPDFEditManager editManager = binding.pdfView.getCPdfReaderView().getEditManager();
            if (editManager.isEditMode()) {
                curEditMode = binding.pdfView.getCPdfReaderView().getLoadType();
                editManager.endEdit();
            }
            binding.pdfToolBar.setVisibility(View.GONE);
            binding.pdfSearchToolBar.setVisibility(View.VISIBLE);
            binding.pdfSearchToolBar.showKeyboard();
        });

        binding.pdfToolBar.setBoTaBtnClickListener(v -> {
            //Show a fragment of the PDF thumbnail and a dialog fragment of the PDF outline list.
            CPDFBotaDialogFragment botaDialogFragment = CPDFBotaDialogFragment.newInstance();
            botaDialogFragment.initWithPDFView(binding.pdfView);
            CPDFBotaFragmentTabs outlineTabs = new CPDFBotaFragmentTabs(CPDFBOTA.OUTLINE, getString(R.string.tools_outlines));
            CPDFBotaFragmentTabs bookmarks = new CPDFBotaFragmentTabs(CPDFBOTA.BOOKMARKS, getString(R.string.tools_bookmarks));
            ArrayList<CPDFBotaFragmentTabs> tabs = new ArrayList<>();
            tabs.add(outlineTabs);
            tabs.add(bookmarks);
            botaDialogFragment.setBotaDialogTabs(tabs);
            botaDialogFragment.show(getSupportFragmentManager(), "boTaDialogFragment");
        });

        binding.pdfToolBar.setThumbnailBtnClickListener(v -> {
            CPDFBotaDialogFragment botaDialogFragment = CPDFBotaDialogFragment.newInstance();
            botaDialogFragment.initWithPDFView(binding.pdfView);
            CPDFBotaFragmentTabs thumbnailTabs = new CPDFBotaFragmentTabs(CPDFBOTA.THUMBNAIL, getString(R.string.tools_thumbnails));
            ArrayList<CPDFBotaFragmentTabs> tabs = new ArrayList<>();
            tabs.add(thumbnailTabs);
            botaDialogFragment.setBotaDialogTabs(tabs);
            botaDialogFragment.show(getSupportFragmentManager(), "boTaDialogFragment");
        });
        binding.pdfToolBar.setMoreBtnClickListener(v -> {
            v.setSelected(true);
            //Show the PDF settings dialog fragment
            CPopupMenuWindow menuWindow = new CPopupMenuWindow(this);
            menuWindow.addItem(R.drawable.tools_ic_preview_settings, R.string.tools_view_setting, v1 -> {
                showDisplaySettings(binding.pdfView);
            });
            menuWindow.addItem(R.drawable.tools_ic_document_info, R.string.tools_document_info, v1 -> {
                showDocumentInfo(binding.pdfView);
            });
            menuWindow.addItem(R.drawable.tools_ic_share, R.string.tools_share, v1 -> {
                sharePDF(binding.pdfView);
            });
            menuWindow.addItem(R.drawable.tools_ic_new_file, R.string.tools_open_document, v1 -> {
                if (hasPermissions(STORAGE_PERMISSIONS)) {
                    selectDocument();
                } else {
                    EasyPermissions.requestPermissions(this, "request permission", REQUEST_EXTERNAL_PERMISSION, STORAGE_PERMISSIONS);
                }
            });
            menuWindow.setOnDismissListener(() -> v.setSelected(false));
            menuWindow.showAsDropDown(v);
        });

        binding.pdfToolBar.setPreviewModeChangeListener((mode) -> {
            binding.pdfToolBar.selectMode(mode);
            screenManager.changeWindowStatus(mode);
            resetContextMenu(binding.pdfView, mode);
            if (mode == CPreviewMode.Viewer) {
                CPDFEditManager editManager = binding.pdfView.getCPdfReaderView().getEditManager();
                if (editManager.isEditMode()) {
                    editManager.endEdit();
                }
            } else {
                CPDFEditManager editManager = binding.pdfView.getCPdfReaderView().getEditManager();
                if (!editManager.isEditMode()) {
                    editManager.beginEdit(CPDFEditType.EDIT_TEXT_IMAGE);
                }
            }
        });
    }

    private void initSearchBar() {
        binding.pdfSearchToolBar.initWithPDFView(binding.pdfView);
        binding.pdfSearchToolBar.onSearchQueryResults(list -> {
            CSearchResultDialogFragment searchResultDialog = new CSearchResultDialogFragment();
            searchResultDialog.show(getSupportFragmentManager(), "searchResultDialogFragment");
            searchResultDialog.setSearchTextInfos(list);
            searchResultDialog.setOnClickSearchItemListener(clickItem -> {
                binding.pdfView.getCPdfReaderView().setDisplayPageIndex(clickItem.page);
                binding.pdfView.getCPdfReaderView().getTextSearcher().searchBegin(clickItem.page, clickItem.textRangeIndex);
                searchResultDialog.dismiss();
            });
        });

        binding.pdfSearchToolBar.setExitSearchListener(() -> {
            if (curEditMode > 0) {
                CPDFEditManager editManager = binding.pdfView.getCPdfReaderView().getEditManager();
                editManager.beginEdit(curEditMode);
                curEditMode = 0;
            }
            binding.pdfToolBar.setVisibility(View.VISIBLE);
            binding.pdfSearchToolBar.setVisibility(View.GONE);
        });
    }

    private void initEditBar() {
        if (binding.pdfView == null || binding.pdfView.getCPdfReaderView() == null) {
            return;
        }
        binding.pdfEditToolBar.initWithPDFView(binding.pdfView);
        binding.pdfEditToolBar.setEditPropertyBtnClickListener((view) -> {
            int type = binding.pdfView.getCPdfReaderView().getSelectAreaType();
            CStyleType styleType = CStyleType.UNKNOWN;
            if (type == SELECT_AREA_TEXT) {
                styleType = CStyleType.EDIT_TEXT;
            } else if (type == SELECT_AREA_IMAGE) {
                styleType = CStyleType.EDIT_IMAGE;
            }
            if (styleType != CStyleType.UNKNOWN) {
                CPDFReaderView readerView = binding.pdfView.getCPdfReaderView();
                CPDFContextMenuHelper menuHelper = (CPDFContextMenuHelper) readerView.getContextMenuShowListener();
                if (menuHelper == null || menuHelper.getReaderView() == null) {
                    return;
                }
                CStyleManager styleManager = new CStyleManager(menuHelper.getEditSelection(), menuHelper.getPageView());
                CAnnotStyle annotStyle = styleManager.getStyle(styleType);
                CStyleDialogFragment dialogFragment = CStyleDialogFragment.newInstance(annotStyle);
                styleManager.setAnnotStyleFragmentListener(dialogFragment);
                styleManager.setDialogHeightCallback(dialogFragment, binding.pdfView.getCPdfReaderView());
                dialogFragment.show(getSupportFragmentManager(), "textPropertyDialogFragment");

                menuHelper.dismissContextMenu();
            }
        });
        binding.pdfView.setEndScrollCallback(() -> {
            binding.pdfEditToolBar.updateUndoRedo();
        });
    }

    private void selectDocument() {
        if (binding.pdfToolBar.getMode() == CPreviewMode.Edit) {
            CPDFReaderView readerView = binding.pdfView.getCPdfReaderView();
            if (readerView != null && readerView.getEditManager() != null && readerView.getEditManager().isEditMode()) {
                readerView.getEditManager().endEdit();
            }
        }
        if (!binding.pdfView.getCPdfReaderView().getPDFDocument().hasChanges()){
            selectDocumentLauncher.launch(CFileUtils.getContentIntent());
            return;
        }
        CAlertDialog alertDialog = CAlertDialog.newInstance(getString(com.compdfkit.tools.R.string.tools_save_title), getString(com.compdfkit.tools.R.string.tools_save_message));
        alertDialog.setConfirmClickListener(v -> {
            //save pdf document
            binding.pdfView.savePDF((filePath, pdfUri) -> {
                alertDialog.dismiss();
                selectDocumentLauncher.launch(CFileUtils.getContentIntent());
            });
        });
        alertDialog.setCancelClickListener(v -> {
            alertDialog.dismiss();
            selectDocumentLauncher.launch(CFileUtils.getContentIntent());
        });
        alertDialog.show(getSupportFragmentManager(), "alertDialog");
    }

    @AfterPermissionGranted(RC_PERMISSION_PERM)
    private void onDoNext() {
        if (!hasPermissions()) {
            EasyPermissions.requestPermissions(this, getString(R.string.edit_app_permission_storage), RC_PERMISSION_PERM, STORAGE_PERMISSIONS);
        }
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        if (requestCode == REQUEST_EXTERNAL_PERMISSION) {
            if (EasyPermissions.hasPermissions(this, android.Manifest.permission.READ_EXTERNAL_STORAGE, Manifest.permission.WRITE_EXTERNAL_STORAGE)) {
                selectDocument();
            }
        }
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == ReplaceEditImageArea) {
            CThreadPoolUtils.getInstance().execute(() -> {
                if (binding.pdfView == null || binding.pdfView.getCPdfReaderView() == null || data == null) {
                    return;
                }
                for (int i = 0; i < binding.pdfView.getCPdfReaderView().getChildCount(); i++) {
                    CPDFPageView pageView = (CPDFPageView) binding.pdfView.getCPdfReaderView().getChildAt(i);
                    if (pageView == null) {
                        continue;
                    }
                    if (pageView.getPageNum() == binding.pdfView.getCPdfReaderView().getPageNum()) {
                        CUriUtil util = new CUriUtil();
                        boolean ret = pageView.operateEditImageArea(CPDFPageView.EditImageFuncType.REPLACE, util.getPath(this, data.getData()));
                        if (ret == false) {
                            Toast.makeText(getApplicationContext(), "replace fail", Toast.LENGTH_LONG).show();
                        }
                        break;
                    }
                }
            });
        } else if (requestCode == AddEditImageArea) {
            if (binding.pdfView == null || binding.pdfView.getCPdfReaderView() == null) {
                return;
            }
            if (data == null) {
                return;
            }
            CThreadPoolUtils.getInstance().execute(() -> {
                for (int i = 0; i < binding.pdfView.getCPdfReaderView().getChildCount(); i++) {
                    CPDFPageView pageView = (CPDFPageView) binding.pdfView.getCPdfReaderView().getChildAt(i);
                    if (pageView == null) {
                        continue;
                    }
                    if (pageView.getPageNum() == binding.pdfView.getCPdfReaderView().getAddImagePage()) {
                        CUriUtil util = new CUriUtil();
                        boolean ret = pageView.addEditImageArea(binding.pdfView.getCPdfReaderView().getAddImagePoint(), util.getPath(this, data.getData()));
                        if (ret == false) {
                            Toast.makeText(getApplicationContext(), "add fail", Toast.LENGTH_LONG).show();
                        }
                        break;
                    }
                }
            });
        }
    }

    @Override
    public void onBackPressed() {
        if (binding.pdfView != null) {
            if (binding.pdfView.getCPdfReaderView() != null && binding.pdfView.getCPdfReaderView().getEditManager() != null) {
                binding.pdfView.getCPdfReaderView().getEditManager().endEdit();
            }
            binding.pdfView.savePDF((filePath, pdfUri) -> super.onBackPressed());
        }
    }
}

==> An_V2.0.3.tar.gz: kmpdfkit_demo/ContentEditor/src/main/java/com/compdfkit/contenteditor/PDFEditSampleActivity.java <==
/**
 * Copyright © 2014-2023 PDF Technologies, Inc. All Rights Reserved.
 * <p>
 * THIS SOURCE CODE AND ANY ACCOMPANYING DOCUMENTATION ARE PROTECTED BY INTERNATIONAL COPYRIGHT LAW
 * AND MAY NOT BE RESOLD OR REDISTRIBUTED. USAGE IS BOUND TO THE ComPDFKit LICENSE AGREEMENT.
 * UNAUTHORIZED REPRODUCTION OR DISTRIBUTION IS SUBJECT TO CIVIL AND CRIMINAL PENALTIES.
 * This notice may not be removed from this file.
 */
package com.compdfkit.contenteditor;

import static com.compdfkit.tools.contenteditor.CEditToolbar.SELECT_AREA_IMAGE;
import static com.compdfkit.tools.contenteditor.CEditToolbar.SELECT_AREA_TEXT;

import android.Manifest;
import android.os.Build;
import android.os.Bundle;
import android.view.View;

import androidx.activity.result.ActivityResultLauncher;

import com.compdfkit.contenteditor.databinding.EditPdfSampleActivityBinding;
import com.compdfkit.core.edit.CPDFEditManager;
import com.compdfkit.core.edit.CPDFEditPage;
import com.compdfkit.tools.common.basic.activity.CBasicPDFActivity;
import com.compdfkit.tools.common.contextmenu.CPDFContextMenuHelper;
import com.compdfkit.tools.common.pdf.config.AnnotationsConfig;
import com.compdfkit.tools.common.utils.CFileUtils;
import com.compdfkit.tools.common.utils.CPermissionUtil;
import com.compdfkit.tools.common.utils.activitycontracts.CSelectPDFDocumentResultContract;
import com.compdfkit.tools.common.utils.dialog.CAlertDialog;
import com.compdfkit.tools.common.utils.task.CExtractAssetFileTask;
import com.compdfkit.tools.common.utils.window.CPopupMenuWindow;
import com.compdfkit.tools.common.views.pdfbota.CPDFBOTA;
import com.compdfkit.tools.common.views.pdfbota.CPDFBotaDialogFragment;
import com.compdfkit.tools.common.views.pdfbota.CPDFBotaFragmentTabs;
import com.compdfkit.tools.common.views.pdfproperties.pdfstyle.CAnnotStyle;
import com.compdfkit.tools.common.views.pdfproperties.pdfstyle.CStyleDialogFragment;
import com.compdfkit.tools.common.views.pdfproperties.pdfstyle.CStyleType;
import com.compdfkit.tools.common.views.pdfproperties.pdfstyle.manager.CStyleManager;
import com.compdfkit.tools.common.views.pdfview.CPreviewMode;
import com.compdfkit.tools.viewer.pdfsearch.CSearchResultDialogFragment;
import com.compdfkit.ui.reader.CPDFReaderView;

import java.util.ArrayList;
import java.util.Arrays;

public class PDFEditSampleActivity extends CBasicPDFActivity {

    /**
     * assets folder pdf file
     */
    public static final String QUICK_START_GUIDE = "ComPDFKit_Sample_File_Android.pdf";

    private EditPdfSampleActivityBinding binding;

    private CEditSampleScreenManager screenManager = new CEditSampleScreenManager();

    private ActivityResultLauncher<Void> selectDocumentLauncher = registerForActivityResult(new CSelectPDFDocumentResultContract(), uri -> {
        if (uri != null) {
            CPDFReaderView readerView = binding.pdfView.getCPdfReaderView();
            if (readerView != null && readerView.getEditManager() != null) {
                readerView.getEditManager().endEdit();
            }
            CFileUtils.takeUriPermission(this, uri);
            binding.pdfEditToolBar.resetStatus();
            setPreviewMode(CPreviewMode.Edit);
            binding.pdfView.openPDF(uri, null, () -> {
                binding.pdfEditToolBar.setEditMode(true);
            });
        }
    });

    int curEditMode = 0;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        binding = EditPdfSampleActivityBinding.inflate(getLayoutInflater());
        setContentView(binding.getRoot());
        screenManager.bind(binding);
        initPDFView();
        initToolbarListener();
        initSearchBar();
        initEditBar();
        onDoNext();
    }

    private void initPDFView() {
        binding.pdfView.setViewMode(CPDFReaderView.ViewMode.PDFEDIT);
        resetContextMenu(binding.pdfView, CPreviewMode.Edit);
        binding.pdfView.getCPdfReaderView().setMinScaleEnable(false);
        //Extract PDF files from the Android assets folder
        CExtractAssetFileTask.extract(this, QUICK_START_GUIDE, QUICK_START_GUIDE, (filePath) -> {
            binding.pdfView.openPDF(filePath);
            binding.pdfEditToolBar.setEditMode(true);
        });
        binding.pdfView.setOnTapMainDocAreaCallback(() -> {
            screenManager.fillScreenChange();
        });
        binding.pdfView.getCPdfReaderView().setMinScaleEnable(false);
        registerFormHelper(binding.pdfView);
    }

    private void initToolbarListener() {
        binding.pdfToolBar.addModes(Arrays.asList(
                CPreviewMode.Viewer,
                CPreviewMode.Edit
        ));
        binding.pdfToolBar.selectMode(CPreviewMode.Edit);
        binding.pdfToolBar.addAction(R.drawable.tools_ic_thumbnail, v -> {
            CPDFBotaDialogFragment botaDialogFragment = CPDFBotaDialogFragment.newInstance();
            botaDialogFragment.initWithPDFView(binding.pdfView);
            CPDFBotaFragmentTabs thumbnailTabs = new CPDFBotaFragmentTabs(CPDFBOTA.THUMBNAIL, getString(R.string.tools_thumbnails));
            ArrayList<CPDFBotaFragmentTabs> tabs = new ArrayList<>();
            tabs.add(thumbnailTabs);
            botaDialogFragment.setBotaDialogTabs(tabs);
            botaDialogFragment.show(getSupportFragmentManager(), "boTaDialogFragment");
        });
        binding.pdfToolBar.addAction(R.drawable.tools_ic_search, v -> {
            CPDFEditManager editManager = binding.pdfView.getCPdfReaderView().getEditManager();
            if (editManager.isEditMode()) {
                curEditMode = binding.pdfView.getCPdfReaderView().getLoadType();
                editManager.endEdit();
            }
            binding.pdfToolBar.setVisibility(View.GONE);
            binding.pdfSearchToolBar.setVisibility(View.VISIBLE);
            binding.pdfSearchToolBar.showKeyboard();
        });
        binding.pdfToolBar.addAction(R.drawable.tools_ic_bookmark, v -> {
            //Show a fragment of the PDF thumbnail and a dialog fragment of the PDF outline list.
            CPDFBotaDialogFragment botaDialogFragment = CPDFBotaDialogFragment.newInstance();
            botaDialogFragment.initWithPDFView(binding.pdfView);
            CPDFBotaFragmentTabs outlineTabs = new CPDFBotaFragmentTabs(CPDFBOTA.OUTLINE, getString(R.string.tools_outlines));
            CPDFBotaFragmentTabs bookmarks = new CPDFBotaFragmentTabs(CPDFBOTA.BOOKMARKS, getString(R.string.tools_bookmarks));
            ArrayList<CPDFBotaFragmentTabs> tabs = new ArrayList<>();
            tabs.add(outlineTabs);
            tabs.add(bookmarks);
            botaDialogFragment.setBotaDialogTabs(tabs);
            botaDialogFragment.show(getSupportFragmentManager(), "boTaDialogFragment");
        });
        binding.pdfToolBar.addAction(R.drawable.tools_ic_more, v -> {
            v.setSelected(true);
            //Show the PDF settings dialog fragment
            CPopupMenuWindow menuWindow = new CPopupMenuWindow(this);
            menuWindow.addItem(R.drawable.tools_ic_preview_settings, R.string.tools_view_setting, v1 -> {
                showDisplaySettings(binding.pdfView);
            });
            menuWindow.addItem(R.drawable.tools_ic_document_info, R.string.tools_document_info, v1 -> {
                showDocumentInfo(binding.pdfView);
            });
            menuWindow.addItem(R.drawable.tools_ic_share, R.string.tools_share, v1 -> {
                sharePDF(binding.pdfView);
            });
            menuWindow.addItem(R.drawable.tools_ic_new_file, R.string.tools_open_document, v1 -> {
                if (CPermissionUtil.hasStoragePermissions(this)) {
                    selectDocument();
                } else {
                    requestStoragePermissions();
                }
            });
            menuWindow.setOnDismissListener(() -> v.setSelected(false));
            menuWindow.showAsDropDown(v);
        });

        binding.pdfToolBar.setPreviewModeChangeListener(this::setPreviewMode);
    }

    private void setPreviewMode(CPreviewMode mode) {
        binding.pdfToolBar.selectMode(mode);
        screenManager.changeWindowStatus(mode);
        resetContextMenu(binding.pdfView, mode);
        if (mode == CPreviewMode.Viewer) {
            binding.pdfView.setViewMode(CPDFReaderView.ViewMode.VIEW);
            CPDFEditManager editManager = binding.pdfView.getCPdfReaderView().getEditManager();
            if (editManager.isEditMode()) {
                editManager.endEdit();
            }
        } else {
            binding.pdfView.setViewMode(CPDFReaderView.ViewMode.PDFEDIT);
            CPDFEditManager editManager = binding.pdfView.getCPdfReaderView().getEditManager();
            if (!editManager.isEditMode()) {
                editManager.beginEdit(CPDFEditPage.LoadTextImage);
            }
        }
    }

    private void requestStoragePermissions() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
            CPermissionUtil.openManageAllFileAppSettings(this);
        } else {
            multiplePermissionResultLauncher.launch(STORAGE_PERMISSIONS, result -> {
                if (CPermissionUtil.hasStoragePermissions(this)) {
                    selectDocument();
                } else {
                    if (!CPermissionUtil.shouldShowRequestPermissionRationale(this, Manifest.permission.WRITE_EXTERNAL_STORAGE)) {
                        showPermissionsRequiredDialog();
                    }
                }
            });
        }
    }

    private void initSearchBar() {
        binding.pdfSearchToolBar.initWithPDFView(binding.pdfView);
        binding.pdfSearchToolBar.onSearchQueryResults(list -> {
            CSearchResultDialogFragment searchResultDialog = new CSearchResultDialogFragment();
            searchResultDialog.show(getSupportFragmentManager(), "searchResultDialogFragment");
            searchResultDialog.setSearchTextInfos(list);
            searchResultDialog.setOnClickSearchItemListener(clickItem -> {
                binding.pdfView.getCPdfReaderView().setDisplayPageIndex(clickItem.page);
                binding.pdfView.getCPdfReaderView().getTextSearcher().searchBegin(clickItem.page, clickItem.textRangeIndex);
                searchResultDialog.dismiss();
            });
        });

        binding.pdfSearchToolBar.setExitSearchListener(() -> {
            if (curEditMode > CPDFEditPage.LoadNone) {
                CPDFEditManager editManager = binding.pdfView.getCPdfReaderView().getEditManager();
                editManager.beginEdit(curEditMode);
                curEditMode = CPDFEditPage.LoadNone;
            }
            binding.pdfToolBar.setVisibility(View.VISIBLE);
            binding.pdfSearchToolBar.setVisibility(View.GONE);
        });
    }

    private void initEditBar() {
        if (binding.pdfView == null || binding.pdfView.getCPdfReaderView() == null) {
            return;
        }
        binding.pdfEditToolBar.initWithPDFView(binding.pdfView);
        binding.pdfEditToolBar.setEditPropertyBtnClickListener((view) -> {
            int type = binding.pdfView.getCPdfReaderView().getSelectAreaType();
            CStyleType styleType = CStyleType.UNKNOWN;
            if (type == SELECT_AREA_TEXT) {
                styleType = CStyleType.EDIT_TEXT;
            } else if (type == SELECT_AREA_IMAGE) {
                styleType = CStyleType.EDIT_IMAGE;
            }
            if (styleType != CStyleType.UNKNOWN) {
                CPDFReaderView readerView = binding.pdfView.getCPdfReaderView();
                CPDFContextMenuHelper menuHelper = (CPDFContextMenuHelper) readerView.getContextMenuShowListener();
                if (menuHelper == null || menuHelper.getReaderView() == null) {
                    return;
                }
                CStyleManager styleManager = new CStyleManager(menuHelper.getEditSelection(), menuHelper.getPageView());
                CAnnotStyle annotStyle = styleManager.getStyle(styleType);
                CStyleDialogFragment dialogFragment = CStyleDialogFragment.newInstance(annotStyle);
                styleManager.setAnnotStyleFragmentListener(dialogFragment);
                styleManager.setDialogHeightCallback(dialogFragment, binding.pdfView.getCPdfReaderView());
                dialogFragment.show(getSupportFragmentManager(), "textPropertyDialogFragment");

                menuHelper.dismissContextMenu();
            }
        });
        binding.pdfEditToolBar.setTools(Arrays.asList(AnnotationsConfig.AnnotationTools.Setting,
                AnnotationsConfig.AnnotationTools.Undo,
                AnnotationsConfig.AnnotationTools.Redo
        ));
        binding.pdfView.setEndScrollCallback(() -> {
            binding.pdfEditToolBar.updateUndoRedo();
        });
    }

    private void selectDocument() {
        if (binding.pdfToolBar.getMode() == CPreviewMode.Edit) {
            CPDFReaderView readerView = binding.pdfView.getCPdfReaderView();
            if (readerView != null && readerView.getEditManager() != null && readerView.getEditManager().isEditMode()) {
                readerView.getEditManager().endEdit();
            }
        }
        if (binding.pdfView.getCPdfReaderView().getPDFDocument() == null) {
            selectDocumentLauncher.launch(null);
            return;
        }
        if (!binding.pdfView.getCPdfReaderView().getPDFDocument().hasChanges()) {
            selectDocumentLauncher.launch(null);
            return;
        }
        CAlertDialog alertDialog = CAlertDialog.newInstance(getString(com.compdfkit.tools.R.string.tools_save_title), getString(com.compdfkit.tools.R.string.tools_save_message));
        alertDialog.setConfirmClickListener(v -> {
            //save pdf document
            binding.pdfView.savePDF((filePath, pdfUri) -> {
                alertDialog.dismiss();
                selectDocumentLauncher.launch(null);
            }, e -> {
                alertDialog.dismiss();
                selectDocumentLauncher.launch(null);
            });
        });
        alertDialog.setCancelClickListener(v -> {
            alertDialog.dismiss();
            selectDocumentLauncher.launch(null);
        });
        alertDialog.show(getSupportFragmentManager(), "alertDialog");
    }

    private void onDoNext() {
        multiplePermissionResultLauncher.launch(STORAGE_PERMISSIONS);
    }

    @Override
    public void onBackPressed() {
        if (binding.pdfView != null) {
            if (binding.pdfView.getCPdfReaderView() != null && binding.pdfView.getCPdfReaderView().getEditManager() != null) {
                binding.pdfView.getCPdfReaderView().getEditManager().endEdit();
            }
            binding.pdfView.savePDF((filePath, pdfUri) -> super.onBackPressed(), e -> super.onBackPressed());
        } else {
            super.onBackPressed();
        }
    }
}
